"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shouldCreateGradient = exports.AutoFunction = exports.GradientContext = void 0;
var tensor_base_1 = require("./tensor_base");
function isTensor(input) {
    return input instanceof tensor_base_1.TensorBase;
}
var GradientContext = /** @class */ (function () {
    function GradientContext(inputs) {
        this.savedTensors = [];
        this.needsInputGradient = inputs.map(function (input) { return isTensor(input) && input.requiresGrad; });
        this.inputsWithGradient = inputs.map(function (input) {
            return isTensor(input) && input.requiresGrad ? input : null;
        });
    }
    GradientContext.prototype.saveForBackward = function () {
        var tensors = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            tensors[_i] = arguments[_i];
        }
        this.savedTensors = tensors;
    };
    return GradientContext;
}());
exports.GradientContext = GradientContext;
var AutoFunction = /** @class */ (function () {
    function AutoFunction() {
    }
    AutoFunction.forward = function (inputs) {
        throw new Error("Do not call forward on AutoFunction directly.");
    };
    AutoFunction.setupContext = function (ctx, inputs, output) {
        throw new Error("Do not call setupContext on AutoFunction directly.");
    };
    AutoFunction.backward = function (ctx, outputGrad) {
        throw new Error("Do not call backward on AutoFunction directly.");
    };
    AutoFunction.apply = function () {
        var inputs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            inputs[_i] = arguments[_i];
        }
        var ctx = new GradientContext(inputs);
        var detachedInputs = inputs.map(function (input) {
            return isTensor(input) ? input.detach() : input;
        });
        var output = this.forward(detachedInputs);
        this.setupContext(ctx, detachedInputs, output);
        output.setGradientFunction(ctx, this.backward);
        return output;
    };
    return AutoFunction;
}());
exports.AutoFunction = AutoFunction;
function shouldCreateGradient() {
    var e_1, _a;
    var inputs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        inputs[_i] = arguments[_i];
    }
    try {
        for (var inputs_1 = __values(inputs), inputs_1_1 = inputs_1.next(); !inputs_1_1.done; inputs_1_1 = inputs_1.next()) {
            var input = inputs_1_1.value;
            if (input.requiresGrad) {
                return true;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (inputs_1_1 && !inputs_1_1.done && (_a = inputs_1.return)) _a.call(inputs_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return false;
}
exports.shouldCreateGradient = shouldCreateGradient;
//# sourceMappingURL=autograd.js.map