"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Device = void 0;
var shape_1 = require("./shape");
var dtype_1 = require("./dtype");
var kernel_1 = require("./kernel");
var kernels_1 = require("./kernels");
var Device = /** @class */ (function () {
    function Device(id, type) {
        this._kernels = {};
        this._id = id;
        this._type = type;
    }
    Object.defineProperty(Device.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Device.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: false,
        configurable: true
    });
    Device.prototype.allocFor = function (shape, dtype) {
        var elementByteSize = (0, dtype_1.dtypeByteSize)(dtype);
        var byteSize = (0, shape_1.shapeSize)(shape) * elementByteSize;
        return this.alloc(byteSize);
    };
    Device.prototype.getKernel = function (name, config) {
        var spec = kernels_1.registry[name];
        if (spec === undefined) {
            throw new Error("Kernel \"".concat(name, "\" not found"));
        }
        var kconfig = (0, kernel_1.getKernelConfig)(spec, config);
        var key = (0, kernel_1.getKernelKey)(spec, kconfig);
        var kernel = this._kernels[key];
        if (kernel === undefined) {
            kernel = this.createKernel(spec, kconfig);
            this._kernels[key] = kernel;
        }
        return kernel;
    };
    return Device;
}());
exports.Device = Device;
//# sourceMappingURL=device.js.map