"use strict";
// "Any sufficiently complicated C or Fortran program contains an ad hoc,
// informally-specified, bug-ridden, slow implementation of half of Common Lisp."
// - Greenspun's Tenth Rule
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.evalCode = exports.compileCode = exports.exprCodeToWebGLShader = exports.exprNodeToWebGLShader = exports.exprNodeToString = exports.substituteIdentifiers = exports.substitute = exports.parseCode = exports.ManifestNumber = void 0;
var ManifestNumber = /** @class */ (function () {
    function ManifestNumber(type, value) {
        this.type = type;
        this.value = value;
    }
    return ManifestNumber;
}());
exports.ManifestNumber = ManifestNumber;
function lexn(code) {
    var tokens = [];
    var n = code.length;
    var i = 0;
    while (i < n) {
        var c = code[i];
        if (c === " " || c === "\t" || c === "\n") {
            i++;
            continue;
        }
        if (c === "+" || c === "-" || c === "*" || c === "/" || c == "," || c == ";" || c == ":" || c == "?" || c == "^" || c == "%" || c == "~" || c == "(" || c == ")" || c == "[" || c == "]" || c == "{" || c == "}" || c == ".") {
            tokens.push(c);
            i++;
            continue;
        }
        if (c == "=" || c == "<" || c == ">" || c == "!") {
            if (i + 1 < n && code[i + 1] == "=") {
                tokens.push(c + "=");
                i += 2;
                continue;
            }
            tokens.push(c);
            i++;
            continue;
        }
        if (c == "&") {
            if (i + 1 < n && code[i + 1] == "&") {
                tokens.push("&&");
                i += 2;
                continue;
            }
            tokens.push("&");
            i++;
            continue;
        }
        if (c == "|") {
            if (i + 1 < n && code[i + 1] == "|") {
                tokens.push("||");
                i += 2;
                continue;
            }
            tokens.push("|");
            i++;
            continue;
        }
        if (c >= "0" && c <= "9") {
            var s = "";
            var hasDecimal = false;
            while (i < n && (code[i] >= "0" && code[i] <= "9" || code[i] === ".")) {
                if (code[i] === ".") {
                    if (hasDecimal) {
                        throw new Error("Invalid number");
                    }
                    hasDecimal = true;
                }
                s += code[i];
                i++;
            }
            var numType = hasDecimal ? "floatAbstract" : "intAbstract";
            tokens.push(new ManifestNumber(numType, parseFloat(s)));
            continue;
        }
        if ((c >= "a" && c <= "z") ||
            (c >= "A" && c <= "Z") ||
            c === "_") {
            var s = "";
            while (i < n &&
                ((code[i] >= "a" && code[i] <= "z") ||
                    (code[i] >= "A" && code[i] <= "Z") ||
                    (code[i] >= "0" && code[i] <= "9") ||
                    code[i] === "_")) {
                s += code[i];
                i++;
            }
            tokens.push(s);
            continue;
        }
        throw new Error("Unexpected character ".concat(c));
    }
    return tokens;
}
function tokenIsIdent(token) {
    if (token instanceof ManifestNumber) {
        return false;
    }
    if (token.length <= 0)
        return false;
    var c = token[0];
    if (!((c >= "a" && c <= "z") || (c >= "A" && c <= "Z") || c === "_")) {
        return false;
    }
    if (c === "if" || c === "else" || c == "return") {
        return false;
    }
    return true;
}
function parseCode(code) {
    if (typeof code === "number") {
        // Detect if code is an integer
        if (code % 1 === 0) {
            return new ManifestNumber("intAbstract", code);
        }
        return new ManifestNumber("floatAbstract", code);
    }
    var tokens = lexn(code);
    var parsePrimary = function (i) {
        if (i >= tokens.length) {
            return null;
        }
        var t = tokens[i];
        if (t instanceof ManifestNumber) {
            return [t, i + 1];
        }
        if (t === "(") {
            var expr_1 = parseExpr(i + 1);
            if (expr_1 === null) {
                throw new Error("Missing expression");
            }
            i = expr_1[1];
            if (i >= tokens.length) {
                throw new Error("Unexpected end of expression");
            }
            var t2 = tokens[i];
            if (typeof t2 !== "string" || t2 !== ")") {
                throw new Error("Expected )");
            }
            return [expr_1[0], i + 1];
        }
        if (!tokenIsIdent(t)) {
            return null;
        }
        var result = [t, i + 1];
        if (i + 1 < tokens.length && tokens[i + 1] === "(") {
            var args = [];
            var j = i + 2;
            while (j < tokens.length) {
                var arg = parseExpr(j);
                if (arg === null) {
                    throw new Error("Missing argument");
                }
                args.push(arg[0]);
                j = arg[1];
                if (j >= tokens.length) {
                    throw new Error("Unexpected end of expression");
                }
                var t2 = tokens[j];
                if (typeof t2 !== "string") {
                    throw new Error("Expected , or )");
                }
                if (t2 === ")") {
                    break;
                }
                if (t2 !== ",") {
                    throw new Error("Expected ,");
                }
                j++;
            }
            var func = result[0];
            args.splice(0, 0, func);
            result = [["apply", args], j + 1];
        }
        return result;
    };
    function parseExpr(i) {
        if (i >= tokens.length) {
            return null;
        }
        return parseConditional(i);
    }
    function parseConditional(i) {
        if (i >= tokens.length) {
            return null;
        }
        var expr = parseLogicalOr(i);
        if (expr === null) {
            return null;
        }
        i = expr[1];
        if (i >= tokens.length) {
            return expr;
        }
        var t = tokens[i];
        if (t !== "?") {
            return expr;
        }
        var expr2 = parseExpr(i + 1);
        if (expr2 === null) {
            throw new Error("Missing expression after ?");
        }
        i = expr2[1];
        if (i >= tokens.length) {
            throw new Error("Unexpected end of conditional expression");
        }
        var t2 = tokens[i];
        if (typeof t2 !== "string" || t2 !== ":") {
            throw new Error("Expected :");
        }
        var expr3 = parseConditional(i + 1);
        if (expr3 === null) {
            throw new Error("Missing expression after : `".concat(code, "`"));
        }
        return [[t, [expr[0], expr2[0], expr3[0]]], expr3[1]];
    }
    function genericParseSeparatedList(parseChild, seps) {
        var e_1, _a;
        var sepIndex = {};
        try {
            for (var seps_1 = __values(seps), seps_1_1 = seps_1.next(); !seps_1_1.done; seps_1_1 = seps_1.next()) {
                var sep = seps_1_1.value;
                sepIndex[sep] = true;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (seps_1_1 && !seps_1_1.done && (_a = seps_1.return)) _a.call(seps_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return function (i) {
            var expr = parseChild(i);
            if (expr === null) {
                return null;
            }
            i = expr[1];
            while (i < tokens.length) {
                var t = tokens[i];
                if (typeof t !== "string") {
                    break;
                }
                if (t in sepIndex) {
                    var expr2 = parseChild(i + 1);
                    if (expr2 === null) {
                        throw new Error("Missing expression after " + t);
                    }
                    expr = [[t, [expr[0], expr2[0]]], expr2[1]];
                    i = expr2[1];
                    continue;
                }
                break;
            }
            return expr;
        };
    }
    var parseMultiplyOrDivide = genericParseSeparatedList(parseUnary, ["*", "/"]);
    var parseAddOrSubtract = genericParseSeparatedList(parseMultiplyOrDivide, ["+", "-"]);
    var parseRelational = genericParseSeparatedList(parseAddOrSubtract, ["<", ">", "<=", ">="]);
    var parseEquality = genericParseSeparatedList(parseRelational, ["==", "!="]);
    var parseLogicalAnd = genericParseSeparatedList(parseEquality, ["&&"]);
    var parseLogicalOr = genericParseSeparatedList(parseLogicalAnd, ["||"]);
    function parseUnary(i) {
        if (i >= tokens.length) {
            return null;
        }
        var t = tokens[i];
        if (typeof t === "string") {
            if (t === "+" || t === "-") {
                var expr_2 = parseUnary(i + 1);
                if (expr_2 === null) {
                    throw new Error("Missing expression");
                }
                if (t === "+") {
                    return expr_2;
                }
                return [["negate", [expr_2[0]]], expr_2[1]];
            }
            if (t === "!") {
                var expr_3 = parseUnary(i + 1);
                if (expr_3 === null) {
                    throw new Error("Missing expression");
                }
                return [[t, [expr_3[0]]], expr_3[1]];
            }
        }
        return parsePrimary(i);
    }
    function parseStatement(i) {
        if (i >= tokens.length) {
            return null;
        }
        // Empty statement?
        if (tokens[i] === ";") {
            // console.log("parse Empty Statement:", i, tokens.slice(i));
            return [["block", []], i + 1];
        }
        // Block statement?
        if (tokens[i] === "{") {
            // console.log("parse Block Statement:", i, tokens.slice(i));
            var j = i + 1;
            if (j >= tokens.length) {
                throw new Error("Missing } at end of input");
            }
            if (tokens[j] === "}") {
                // console.log("block statement empty");
                return [["block", []], j + 1];
            }
            var statements = parseStatements(j);
            if (statements === null) {
                throw new Error("Missing statements after {");
            }
            j = statements[1];
            // console.log("block statements end:", j, tokens[j]);
            if (tokens[j] !== "}") {
                throw new Error("Missing } (".concat(tokens, ")"));
            }
            // console.log("block statement end:", j + 1, tokens[j + 1]);
            var statementsNode = statements[0];
            if (statementsNode instanceof Array && statementsNode[0] === "statements") {
                return [["block", statementsNode[1]], j + 1];
            }
            else {
                return [["block", [statementsNode]], j + 1];
            }
        }
        // If statement?
        if (tokens[i] === "if") {
            // console.log("parse If Statement:", i, tokens.slice(i));
            var j = i + 1;
            if (j >= tokens.length) {
                throw new Error("Missing ( after if");
            }
            if (tokens[j] !== "(") {
                throw new Error("Missing ( after if");
            }
            var condExpr = parseExpr(j + 1);
            if (condExpr === null) {
                throw new Error("Missing condition after if");
            }
            j = condExpr[1];
            if (j >= tokens.length || tokens[j] !== ")") {
                throw new Error("Missing ) after if condition");
            }
            var thenExpr = parseStatement(j + 1);
            if (thenExpr === null) {
                throw new Error("Missing then after if condition");
            }
            j = thenExpr[1];
            if (j >= tokens.length || tokens[j] !== "else") {
                return [["if", [condExpr[0], thenExpr[0]]], j];
            }
            var elseExpr = parseStatement(j + 1);
            if (elseExpr === null) {
                throw new Error("Missing else after if condition");
            }
            j = elseExpr[1];
            return [["if", [condExpr[0], thenExpr[0], elseExpr[0]]], j];
        }
        // var statement?
        if (tokens[i] === "var") {
            // console.log("parse Var Statement:", i, tokens.slice(i));
            var name_1 = tokens[i + 1];
            if (typeof name_1 !== "string") {
                throw new Error("Missing variable name after var");
            }
            var varEquals = tokens[i + 2];
            if (varEquals !== "=") {
                throw new Error("Missing = after var");
            }
            var varInit = parseExpr(i + 3);
            if (varInit === null) {
                throw new Error("Missing initial value after var");
            }
            return [["var", [name_1, varInit[0]]], varInit[1]];
        }
        // return statement?
        if (tokens[i] === "return") {
            // console.log("parse Return Statement:", i, tokens.slice(i));
            var returnExpr = parseExpr(i + 1);
            if (returnExpr === null) {
                throw new Error("Missing expression after return");
            }
            return [["return", [returnExpr[0]]], returnExpr[1]];
        }
        // Expression statement?
        var expr = parseExpr(i);
        if (expr === null) {
            // console.log("Failed to parse statement");
            return null;
        }
        // console.log("parse Expression Statement:", i, tokens.slice(i), expr);
        i = expr[1];
        if (i < tokens.length && tokens[i] === "=") {
            var expr2 = parseExpr(i + 1);
            if (expr2 === null) {
                throw new Error("Missing right hand side of assignment");
            }
            return [["assign", [expr[0], expr2[0]]], expr2[1]];
        }
        return expr;
    }
    function parseStatements(i) {
        if (i >= tokens.length) {
            return null;
        }
        // console.log("parseStatements:", i, tokens.slice(i));
        var expr = parseStatement(i);
        if (expr === null) {
            return null;
        }
        i = expr[1];
        if (i >= tokens.length) {
            return expr;
        }
        var children = [expr[0]];
        while (i < tokens.length) {
            var t = tokens[i];
            if (t !== ";") {
                break;
            }
            // console.log("got semi-colon at", i, tokens.slice(i));
            i += 1;
            var expr2 = parseStatement(i);
            if (expr2 === null) {
                // console.log("no more statements");
                break;
            }
            children.push(expr2[0]);
            i = expr2[1];
        }
        if (children.length === 1) {
            return [children[0], i];
        }
        // console.log("parsed statements", i, tokens.slice(i));
        return [["statements", children], i];
    }
    var expr = parseStatements(0);
    if (expr === null) {
        throw new Error("Missing expression");
    }
    if (expr[1] < tokens.length) {
        throw new Error("Unexpected token '".concat(tokens[expr[1]], "' after parsing ").concat(JSON.stringify(expr[0]), " from tokens [").concat(tokens.slice(0, expr[1] + 1), "]"));
    }
    return expr[0];
}
exports.parseCode = parseCode;
function substitute(ast, match, replace) {
    var e_2, _a;
    if (ast instanceof ManifestNumber) {
        return ast;
    }
    if (typeof ast === "string") {
        return ast;
    }
    var newChildren = [];
    try {
        for (var _b = __values(ast[1]), _c = _b.next(); !_c.done; _c = _b.next()) {
            var child = _c.value;
            newChildren.push(substitute(child, match, replace));
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_2) throw e_2.error; }
    }
    ast = [ast[0], newChildren];
    if (match(ast)) {
        ast = replace(ast);
    }
    return ast;
}
exports.substitute = substitute;
function substituteIdentifiers(ast, subs) {
    var e_3, _a;
    if (ast instanceof ManifestNumber) {
        return ast;
    }
    if (typeof ast === "string") {
        if (ast in subs) {
            return subs[ast];
        }
        return ast;
    }
    var newChildren = [];
    try {
        for (var _b = __values(ast[1]), _c = _b.next(); !_c.done; _c = _b.next()) {
            var child = _c.value;
            newChildren.push(substituteIdentifiers(child, subs));
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return [ast[0], newChildren];
}
exports.substituteIdentifiers = substituteIdentifiers;
function exprNodeToString(ast) {
    if (ast instanceof ManifestNumber) {
        var s = "";
        if (ast.type === "floatAbstract") {
            s = ast.value.toString();
            if (s.indexOf(".") === -1) {
                s += "f";
            }
        }
        else {
            s = ast.value.toString();
        }
        return s;
    }
    if (typeof ast === "string") {
        return ast;
    }
    var nodeType = ast[0];
    switch (nodeType) {
        case "+":
            return "(".concat(exprNodeToString(ast[1][0]), " + ").concat(exprNodeToString(ast[1][1]), ")");
        case "-":
            return "(".concat(exprNodeToString(ast[1][0]), " - ").concat(exprNodeToString(ast[1][1]), ")");
        case "*":
            return "(".concat(exprNodeToString(ast[1][0]), " * ").concat(exprNodeToString(ast[1][1]), ")");
        case "/":
            return "(".concat(exprNodeToString(ast[1][0]), " / ").concat(exprNodeToString(ast[1][1]), ")");
        case "%":
            return "(".concat(exprNodeToString(ast[1][0]), " % ").concat(exprNodeToString(ast[1][1]), ")");
        case "==":
            return "(".concat(exprNodeToString(ast[1][0]), " == ").concat(exprNodeToString(ast[1][1]), ")");
        case "!=":
            return "(".concat(exprNodeToString(ast[1][0]), " != ").concat(exprNodeToString(ast[1][1]), ")");
        case "<":
            return "(".concat(exprNodeToString(ast[1][0]), " < ").concat(exprNodeToString(ast[1][1]), ")");
        case ">":
            return "(".concat(exprNodeToString(ast[1][0]), " > ").concat(exprNodeToString(ast[1][1]), ")");
        case "<=":
            return "(".concat(exprNodeToString(ast[1][0]), " <= ").concat(exprNodeToString(ast[1][1]), ")");
        case ">=":
            return "(".concat(exprNodeToString(ast[1][0]), " >= ").concat(exprNodeToString(ast[1][1]), ")");
        case "&&":
            return "(".concat(exprNodeToString(ast[1][0]), " && ").concat(exprNodeToString(ast[1][1]), ")");
        case "||":
            return "(".concat(exprNodeToString(ast[1][0]), " || ").concat(exprNodeToString(ast[1][1]), ")");
        case "?":
            return "(".concat(exprNodeToString(ast[1][0]), " ? ").concat(exprNodeToString(ast[1][1]), " : ").concat(exprNodeToString(ast[1][2]), ")");
        case "apply":
            var f = ast[1][0];
            var fstr = exprNodeToString(f);
            var args = ast[1].slice(1);
            var argstrs = args.map(exprNodeToString);
            return "".concat(fstr, "(").concat(argstrs.join(", "), ")");
        case "block":
            return "{ ".concat(ast[1].map(exprNodeToString).join("; "), " }");
        case "assign":
            var lhs = ast[1][0];
            var rhs = ast[1][1];
            return "".concat(exprNodeToString(lhs), " = ").concat(exprNodeToString(rhs));
        case "if":
            var cond = ast[1][0];
            var then = ast[1][1];
            var else_ = ast[1][2];
            return "if (".concat(exprNodeToString(cond), ") { ").concat(exprNodeToString(then), " } else { ").concat(exprNodeToString(else_), " }");
        case "negate":
            return "(-".concat(exprNodeToString(ast[1][0]), ")");
        case "statements":
            return ast[1].map(exprNodeToString).join("; ");
        case "return":
            return "return ".concat(exprNodeToString(ast[1][0]));
        case "var":
            return "var ".concat(ast[1][0], " = ").concat(exprNodeToString(ast[1][1]));
        default:
            throw new Error("Unknown AST node type when printing: ".concat(ast[0]));
    }
}
exports.exprNodeToString = exprNodeToString;
function exprNodeToWebGLShader(ast) {
    // Convert conditional into select
    ast = substitute(ast, function (node) {
        if (!(node instanceof Array)) {
            return false;
        }
        if (node[0] === "?") {
            return true;
        }
        return false;
    }, function (node) {
        node = node;
        var cond = node[1][0];
        var then = node[1][1];
        var else_ = node[1][2];
        return ["apply", ["select", else_, then, cond]];
    });
    return exprNodeToString(ast);
}
exports.exprNodeToWebGLShader = exprNodeToWebGLShader;
function exprCodeToWebGLShader(code, identifierSubs) {
    var ast = parseCode(code);
    if (identifierSubs) {
        ast = substituteIdentifiers(ast, identifierSubs);
    }
    return exprNodeToWebGLShader(ast);
}
exports.exprCodeToWebGLShader = exprCodeToWebGLShader;
/*== COMPILER ==*/
var OP_CONSTANT = 0;
var OP_READ = 1;
var OP_ADD = 2;
var OP_SUBTRACT = 3;
var OP_MULTIPLY = 4;
var OP_DIVIDE = 5;
function compileCode(code) {
    if (typeof code === "number") {
        return function () { return code; };
    }
    var expr = parseCode(code);
    // console.log("parsed", expr);
    var instructions = [];
    function emit(node) {
        if (node instanceof ManifestNumber) {
            instructions.push([OP_CONSTANT, node.value]);
            return;
        }
        if (typeof node === "string") {
            instructions.push([OP_READ, node]);
            return;
        }
        if (node[0] === "+") {
            emit(node[1][0]);
            emit(node[1][1]);
            instructions.push([OP_ADD, null]);
            return;
        }
        if (node[0] === "-") {
            emit(node[1][0]);
            emit(node[1][1]);
            instructions.push([OP_SUBTRACT, null]);
            return;
        }
        if (node[0] === "*") {
            emit(node[1][0]);
            emit(node[1][1]);
            instructions.push([OP_MULTIPLY, null]);
            return;
        }
        if (node[0] === "/") {
            emit(node[1][0]);
            emit(node[1][1]);
            instructions.push([OP_DIVIDE, null]);
            return;
        }
        throw new Error("Unknown node type ".concat(node));
    }
    emit(expr);
    // console.log("ops", ops);
    return function (env) {
        var e_4, _a;
        // Evaluate the ops
        var stack = [];
        try {
            for (var instructions_1 = __values(instructions), instructions_1_1 = instructions_1.next(); !instructions_1_1.done; instructions_1_1 = instructions_1.next()) {
                var ins = instructions_1_1.value;
                var op = ins[0];
                if (op === 0) {
                    stack.push(ins[1]);
                }
                else if (op == 1) {
                    stack.push(env[ins[1]]);
                }
                else {
                    var b = stack.pop();
                    var a = stack.pop();
                    switch (op) {
                        case 2:
                            stack.push(a + b);
                            break;
                        case 3:
                            stack.push(a - b);
                            break;
                        case 4:
                            stack.push(a * b);
                            break;
                        case 5:
                            stack.push(a / b);
                            break;
                    }
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (instructions_1_1 && !instructions_1_1.done && (_a = instructions_1.return)) _a.call(instructions_1);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return stack[0];
    };
}
exports.compileCode = compileCode;
function evalCode(input, env) {
    if (typeof input === "number") {
        return input;
    }
    var c = compileCode(input);
    return c(env);
}
exports.evalCode = evalCode;
//# sourceMappingURL=expr.js.map