"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var e_1, _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.getKernelJavaScriptCode = exports.getKernelShaderCode = exports.shaderTypeToDtype = exports.getShaderTypeElementByteSize = exports.getKernelKey = exports.getKernelConfig = exports.Kernel = void 0;
var expr_1 = require("./expr");
var opgen_1 = require("./opgen");
var Kernel = /** @class */ (function () {
    function Kernel(spec, config, device) {
        this._key = getKernelKey(spec, config);
        this._device = device;
        this._spec = spec;
        this._config = config;
        this._workgroupCountXFunc = (0, expr_1.compileCode)(spec.workgroupCount[0]);
        this._workgroupCountYFunc = (0, expr_1.compileCode)(spec.workgroupCount[1]);
        this._workgroupCountZFunc = (0, expr_1.compileCode)(spec.workgroupCount[2]);
        this._outputSizeFuncs = [];
        for (var i = 0; i < this._spec.outputs.length; i++) {
            var outputSpec = this._spec.outputs[i];
            var outputElementCount = (0, expr_1.compileCode)(outputSpec.size);
            this._outputSizeFuncs.push(outputElementCount);
        }
    }
    Object.defineProperty(Kernel.prototype, "key", {
        get: function () {
            return this._key;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Kernel.prototype, "spec", {
        get: function () {
            return this._spec;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Kernel.prototype, "config", {
        get: function () {
            return this._config;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Kernel.prototype, "device", {
        get: function () {
            return this._device;
        },
        enumerable: false,
        configurable: true
    });
    Kernel.prototype.getRunEnv = function (parameters) {
        var env = {};
        for (var i = 0; i < this._spec.config.length; i++) {
            var configSpec = this._spec.config[i];
            var configValue = this._config[i];
            env[configSpec.name] = configValue;
        }
        for (var i = 0; i < this.spec.parameters.length; i++) {
            var param = this.spec.parameters[i];
            var paramValue = parameters[param.name];
            if (paramValue === undefined) {
                throw new Error("Missing parameter ".concat(param.name));
            }
            env[param.name] = paramValue;
        }
        return env;
    };
    Kernel.prototype.getWorkgroupCounts = function (env) {
        return [
            Math.ceil(this._workgroupCountXFunc(env)),
            Math.ceil(this._workgroupCountYFunc(env)),
            Math.ceil(this._workgroupCountZFunc(env)),
        ];
    };
    return Kernel;
}());
exports.Kernel = Kernel;
function getKernelConfig(spec, config) {
    var configValues = [];
    for (var i = 0; i < spec.config.length; i++) {
        var configSpec = spec.config[i];
        var configValue = config[configSpec.name];
        if (configValue === undefined) {
            throw new Error("Missing config value \"".concat(configSpec.name, "\" for kernel \"").concat(spec.name, "\""));
        }
        configValues.push(configValue);
    }
    return configValues;
}
exports.getKernelConfig = getKernelConfig;
function getKernelKey(spec, config) {
    var keyParts = [spec.name];
    for (var i = 0; i < spec.config.length; i++) {
        var configSpec = spec.config[i];
        var configValue = config[i];
        keyParts.push("".concat(configSpec.name, "=").concat(configValue));
    }
    return keyParts.join(",");
}
exports.getKernelKey = getKernelKey;
function getShaderTypeElementByteSize(shaderType) {
    switch (shaderType) {
        case "f32":
        case "i32":
        case "u32":
        case "array<f32>":
        case "array<i32>":
        case "array<u32>":
            return 4;
        case "u8":
        case "array<u8>":
            return 1;
        default:
            throw new Error("Unknown shader type ".concat(shaderType));
    }
}
exports.getShaderTypeElementByteSize = getShaderTypeElementByteSize;
function shaderTypeToDtype(shaderType) {
    switch (shaderType) {
        case "f32":
        case "array<f32>":
            return "float32";
        case "i32":
        case "array<i32>":
            return "int32";
        case "u32":
        case "array<u32>":
            return "uint32";
        case "u8":
        case "array<u8>":
            return "uint8";
        default:
            throw new Error("Unknown shader type ".concat(shaderType));
    }
}
exports.shaderTypeToDtype = shaderTypeToDtype;
function getIdentifiers(code) {
    var identifierRegex = /[a-zA-Z_][a-zA-Z0-9_]*/g;
    var identifiers = new Set();
    var match;
    while ((match = identifierRegex.exec(code)) !== null) {
        identifiers.add(match[0]);
    }
    return Array.from(identifiers);
}
function configShader(spec, config) {
    var e_2, _a;
    var substituions = [];
    var env = {};
    for (var i = 0; i < spec.config.length; i++) {
        var configSpec = spec.config[i];
        var configValue = config[i];
        substituions.push([configSpec.name, configValue.toString()]);
        env[configSpec.name] = configValue;
    }
    var result = spec.shader.trim();
    try {
        for (var substituions_1 = __values(substituions), substituions_1_1 = substituions_1.next(); !substituions_1_1.done; substituions_1_1 = substituions_1.next()) {
            var _b = __read(substituions_1_1.value, 2), key = _b[0], value = _b[1];
            result = result.replace(new RegExp("\\$\\$".concat(key, "\\$\\$"), "g"), value);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (substituions_1_1 && !substituions_1_1.done && (_a = substituions_1.return)) _a.call(substituions_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return [result, env];
}
function getKernelShaderCode(spec, config) {
    var _a = __read(configShader(spec, config), 2), configdShader = _a[0], env = _a[1];
    var shaderCodeParts = ["// " + spec.name + " kernel"];
    shaderCodeParts.push("struct ".concat(spec.name, "Parameters {"));
    for (var i = 0; i < spec.parameters.length; i++) {
        var parameter = spec.parameters[i];
        shaderCodeParts.push("    ".concat(parameter.name, ": ").concat(parameter.shaderType, ","));
    }
    shaderCodeParts.push("}");
    var bindingIndex = 0;
    for (var i = 0; i < spec.inputs.length; i++, bindingIndex++) {
        var input = spec.inputs[i];
        shaderCodeParts.push("@group(0) @binding(".concat(bindingIndex, ") var<storage, read> ").concat(input.name, ": ").concat(input.shaderType, ";"));
    }
    for (var i = 0; i < spec.outputs.length; i++, bindingIndex++) {
        var output = spec.outputs[i];
        shaderCodeParts.push("@group(0) @binding(".concat(bindingIndex, ") var<storage, read_write> ").concat(output.name, ": ").concat(output.shaderType, ";"));
    }
    shaderCodeParts.push("@group(0) @binding(".concat(bindingIndex, ") var<storage, read> parameters: ").concat(spec.name, "Parameters;"));
    var workgroupSizeX = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[0], env));
    var workgroupSizeY = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[1], env));
    var workgroupSizeZ = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[2], env));
    shaderCodeParts.push("@compute @workgroup_size(".concat(workgroupSizeX, ", ").concat(workgroupSizeY, ", ").concat(workgroupSizeZ, ")"));
    shaderCodeParts.push("fn main(");
    var head = "";
    if (configdShader.includes("global_id")) {
        shaderCodeParts.push("    ".concat(head, "@builtin(global_invocation_id) global_id: vec3u"));
        head = ", ";
    }
    if (configdShader.includes("local_id")) {
        shaderCodeParts.push("    ".concat(head, "@builtin(local_invocation_id) local_id: vec3u"));
        head = ", ";
    }
    shaderCodeParts.push("    ) {");
    shaderCodeParts.push("    " + configdShader);
    shaderCodeParts.push("}");
    var shaderCode = shaderCodeParts.join("\n");
    // console.log(shaderCode);
    return shaderCode;
}
exports.getKernelShaderCode = getKernelShaderCode;
var javaScriptSubstitutions = [
    ["(\\d+)u", "$1"],
    ["(\\d+)f", "$1"],
    ["(\\d+\\.\\d+)f", "$1"],
    ["global_id\\.x", "global_id_x"],
    ["global_id\\.y", "global_id_y"],
    ["global_id\\.z", "global_id_z"],
    ["local_id\\.x", "local_id_x"],
    ["local_id\\.y", "local_id_y"],
    ["local_id\\.z", "local_id_z"],
    ["workgroupBarrier\\s*\\(\\s*\\)\\s*;", "yield \"workgroupBarrier\";"],
    ["storageBarrier\\s*\\(\\s*\\)\\s*;", "yield \"storageBarrier\";"],
].map(function (_a) {
    var _b = __read(_a, 2), regex = _b[0], replacement = _b[1];
    return [new RegExp(regex, "g"), replacement];
});
try {
    // Add all the Math. functions to the substitution list
    for (var _b = __values(Object.getOwnPropertyNames(Math)), _c = _b.next(); !_c.done; _c = _b.next()) {
        var name_1 = _c.value;
        if (typeof Math[name_1] === "function") {
            javaScriptSubstitutions.push([new RegExp("\\b".concat(name_1, "\\("), "g"), "Math.".concat(name_1, "(")]);
        }
    }
}
catch (e_1_1) { e_1 = { error: e_1_1 }; }
finally {
    try {
        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
    }
    finally { if (e_1) throw e_1.error; }
}
var javaScriptGlobalFunctions = {
    exp2: "function exp2(x) { return Math.pow(2, x); }",
    fract: "function fract(x) { return x - Math.floor(x); }",
    select: "function select(falseValue, trueValue, condition) { return condition ? trueValue : falseValue; }",
};
function getKernelJavaScriptCode(spec, config) {
    var e_3, _a, e_4, _b, e_5, _c;
    var _d = __read(configShader(spec, config), 2), configdShader = _d[0], env = _d[1];
    var usesGlobalId = configdShader.includes("global_id");
    var usesLocalId = configdShader.includes("local_id");
    var usesBarrier = configdShader.includes("workgroupBarrier") ||
        configdShader.includes("storageBarrier");
    // Build up the body of the kernel function
    var jsCode = configdShader;
    try {
        for (var javaScriptSubstitutions_1 = __values(javaScriptSubstitutions), javaScriptSubstitutions_1_1 = javaScriptSubstitutions_1.next(); !javaScriptSubstitutions_1_1.done; javaScriptSubstitutions_1_1 = javaScriptSubstitutions_1.next()) {
            var _e = __read(javaScriptSubstitutions_1_1.value, 2), regex = _e[0], replacement = _e[1];
            jsCode = jsCode.replace(regex, replacement);
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (javaScriptSubstitutions_1_1 && !javaScriptSubstitutions_1_1.done && (_a = javaScriptSubstitutions_1.return)) _a.call(javaScriptSubstitutions_1);
        }
        finally { if (e_3) throw e_3.error; }
    }
    // Find needed functions
    var identifiers = getIdentifiers(jsCode);
    var neededFunctions = new Set();
    try {
        for (var identifiers_1 = __values(identifiers), identifiers_1_1 = identifiers_1.next(); !identifiers_1_1.done; identifiers_1_1 = identifiers_1.next()) {
            var identifier = identifiers_1_1.value;
            if (identifier in javaScriptGlobalFunctions) {
                neededFunctions.add(identifier);
            }
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (identifiers_1_1 && !identifiers_1_1.done && (_b = identifiers_1.return)) _b.call(identifiers_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    var neededFunctionsArray = Array.from(neededFunctions);
    // Write the whole function
    var w = new opgen_1.CodeWriter();
    var params = [];
    var bindingIndex = 0;
    for (var i = 0; i < spec.inputs.length; i++, bindingIndex++) {
        var input = spec.inputs[i];
        params.push(input.name);
    }
    for (var i = 0; i < spec.outputs.length; i++, bindingIndex++) {
        var output = spec.outputs[i];
        params.push(output.name);
    }
    params.push("parameters");
    params.push("workgroupCountX");
    params.push("workgroupCountY");
    params.push("workgroupCountZ");
    var workgroupSizeX = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[0], env));
    var workgroupSizeY = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[1], env));
    var workgroupSizeZ = Math.ceil((0, expr_1.evalCode)(spec.workgroupSize[2], env));
    w.writeLine("((".concat(params.join(", "), ") => {"));
    w.indent();
    try {
        // Write dependent functions
        for (var neededFunctionsArray_1 = __values(neededFunctionsArray), neededFunctionsArray_1_1 = neededFunctionsArray_1.next(); !neededFunctionsArray_1_1.done; neededFunctionsArray_1_1 = neededFunctionsArray_1.next()) {
            var neededFunction = neededFunctionsArray_1_1.value;
            w.writeLine(javaScriptGlobalFunctions[neededFunction]);
        }
    }
    catch (e_5_1) { e_5 = { error: e_5_1 }; }
    finally {
        try {
            if (neededFunctionsArray_1_1 && !neededFunctionsArray_1_1.done && (_c = neededFunctionsArray_1.return)) _c.call(neededFunctionsArray_1);
        }
        finally { if (e_5) throw e_5.error; }
    }
    // Write the kernel function
    var kernelParams = [];
    if (usesGlobalId) {
        kernelParams.push("global_id_x");
        kernelParams.push("global_id_y");
        kernelParams.push("global_id_z");
    }
    if (usesLocalId) {
        kernelParams.push("local_id_x");
        kernelParams.push("local_id_y");
        kernelParams.push("local_id_z");
    }
    var kernelParamsString = kernelParams.join(", ");
    w.writeLine("function".concat(usesBarrier ? "*" : "", " ").concat(spec.name, "Kernel(").concat(kernelParamsString, ") {"));
    w.indent();
    w.writeLine(jsCode);
    w.dedent();
    w.writeLine("}");
    // for (let p of params) {
    //     w.writeLine(`console.log("param", "${p}", typeof ${p}, ${p});`);
    // }
    var workgroupSize = workgroupSizeX * workgroupSizeY * workgroupSizeZ;
    if (usesBarrier) {
        w.writeLine("var barriers = new Array(".concat(workgroupSize, ");"));
    }
    w.writeLine("for (let wgZ = 0; wgZ < workgroupCountZ; wgZ++) {");
    w.indent();
    w.writeLine("for (let wgY = 0; wgY < workgroupCountY; wgY++) {");
    w.indent();
    w.writeLine("for (let group_id_x = 0; group_id_x < workgroupCountX; group_id_x++) {");
    w.indent();
    w.writeLine("const globalStartX = group_id_x * ".concat(workgroupSizeX, ";"));
    w.writeLine("const globalEndX = globalStartX + ".concat(workgroupSizeX, ";"));
    w.writeLine("const globalStartY = wgY * ".concat(workgroupSizeY, ";"));
    w.writeLine("const globalEndY = globalStartY + ".concat(workgroupSizeY, ";"));
    w.writeLine("const globalStartZ = wgZ * ".concat(workgroupSizeZ, ";"));
    w.writeLine("const globalEndZ = globalStartZ + ".concat(workgroupSizeZ, ";"));
    if (usesBarrier) {
        w.writeLine("let allDone = true;");
    }
    w.writeLine("for (let global_id_z = globalStartZ; global_id_z < globalEndZ; global_id_z++) {");
    w.indent();
    w.writeLine("for (let global_id_y = globalStartY; global_id_y < globalEndY; global_id_y++) {");
    w.indent();
    w.writeLine("for (let global_id_x = globalStartX; global_id_x < globalEndX; global_id_x++) {");
    w.indent();
    if (usesLocalId) {
        w.writeLine("const local_id_x = global_id_x - globalStartX;");
        w.writeLine("const local_id_y = global_id_y - globalStartY;");
        w.writeLine("const local_id_z = global_id_z - globalStartZ;");
    }
    if (usesBarrier) {
        w.writeLine("const local_index = local_id_x + local_id_y * ".concat(workgroupSizeX, " + local_id_z * ").concat(workgroupSizeX * workgroupSizeY, ";"));
        w.writeLine("const barrier = ".concat(spec.name, "Kernel(").concat(kernelParamsString, ");"));
        w.writeLine("const firstBarrierValue = barrier.next();");
        w.writeLine("allDone = allDone && firstBarrierValue.done;");
        w.writeLine("barriers[local_index] = barrier;");
    }
    else {
        w.writeLine("".concat(spec.name, "Kernel(").concat(kernelParamsString, ");"));
    }
    w.dedent();
    w.writeLine("}");
    w.dedent();
    w.writeLine("}");
    w.dedent();
    w.writeLine("}");
    if (usesBarrier) {
        w.writeLine("while (!allDone) {");
        w.indent();
        w.writeLine("allDone = true;");
        w.writeLine("for (let local_index = 0; local_index < ".concat(workgroupSize, "; local_index++) {"));
        w.indent();
        w.writeLine("const barrier = barriers[local_index];");
        w.writeLine("const barrierValue = barrier.next();");
        w.writeLine("allDone = allDone && barrierValue.done;");
        w.dedent();
        w.writeLine("}");
        w.dedent();
        w.writeLine("}");
    }
    w.dedent();
    w.writeLine("}");
    w.dedent();
    w.writeLine("}");
    w.dedent();
    w.writeLine("}");
    w.dedent();
    w.writeLine("})");
    var code = w.toString();
    // console.log(code);
    return code;
}
exports.getKernelJavaScriptCode = getKernelJavaScriptCode;
//# sourceMappingURL=kernel.js.map