"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KernelWebGPU = void 0;
var kernel_1 = require("./kernel");
var KernelWebGPU = /** @class */ (function (_super) {
    __extends(KernelWebGPU, _super);
    function KernelWebGPU(spec, config, device) {
        var _this = _super.call(this, spec, config, device) || this;
        var gpuDevice = device.gpuDevice;
        if (!gpuDevice) {
            throw new Error("Cannot create a GPU kernel without a GPU device");
        }
        _this._gpuDevice = gpuDevice;
        var bindGroupLayoutEntries = [];
        var bindingIndex = 0;
        for (var i = 0; i < spec.inputs.length; i++, bindingIndex++) {
            bindGroupLayoutEntries.push({
                binding: bindingIndex,
                visibility: GPUShaderStage.COMPUTE,
                buffer: {
                    type: "read-only-storage",
                },
            });
        }
        for (var i = 0; i < spec.outputs.length; i++, bindingIndex++) {
            bindGroupLayoutEntries.push({
                binding: bindingIndex,
                visibility: GPUShaderStage.COMPUTE,
                buffer: {
                    type: "storage",
                },
            });
        }
        bindGroupLayoutEntries.push({
            binding: bindingIndex,
            visibility: GPUShaderStage.COMPUTE,
            buffer: {
                type: "read-only-storage",
            },
        });
        _this._bindGroupLayout = gpuDevice.createBindGroupLayout({
            entries: bindGroupLayoutEntries,
        });
        _this._shaderCode = (0, kernel_1.getKernelShaderCode)(spec, config);
        var shaderModule = gpuDevice.createShaderModule({
            code: _this._shaderCode,
        });
        _this._computePipeline = gpuDevice.createComputePipeline({
            layout: gpuDevice.createPipelineLayout({
                bindGroupLayouts: [_this._bindGroupLayout],
            }),
            compute: {
                module: shaderModule,
                entryPoint: "main",
            },
        });
        return _this;
    }
    KernelWebGPU.prototype.run = function (inputs, parameters, outputs) {
        // console.log("run gpu kernel", this.key);
        var e_1, _a;
        var _this = this;
        // Build the parameter environment
        var env = this.getRunEnv(parameters);
        var paramsBufferSize = 0;
        for (var i = 0; i < this.spec.parameters.length; i++) {
            var param = this.spec.parameters[i];
            paramsBufferSize += (0, kernel_1.getShaderTypeElementByteSize)(param.shaderType);
        }
        // Get input buffers with storage usage
        var storageInputs = this.spec.inputs.map(function (input, i) {
            return _this.getStorageInputBuffer(input, inputs[i] ? inputs[i] : null, i, env);
        });
        // Get output buffers with storage usage
        var storageOutputs = this.spec.outputs.map(function (output, i) {
            return _this.getStorageOutputBuffer(output, outputs ? outputs[i] : null, i, env);
        });
        // Build the params buffer
        var paramsBuffer = this._gpuDevice.createBuffer({
            mappedAtCreation: true,
            size: paramsBufferSize,
            usage: GPUBufferUsage.STORAGE,
        });
        var paramsArrayBuffer = paramsBuffer.getMappedRange();
        try {
            for (var _b = __values(["u32", "f32"]), _c = _b.next(); !_c.done; _c = _b.next()) {
                var paramDtype = _c.value;
                var paramsArray = new (paramDtype === "u32" ? Uint32Array : Float32Array)(paramsArrayBuffer);
                for (var i = 0; i < this.spec.parameters.length; i++) {
                    var param = this.spec.parameters[i];
                    if (param.shaderType === paramDtype) {
                        paramsArray[i] = +env[param.name];
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        paramsBuffer.unmap();
        // Bind the buffers
        var bindGroup = this.createBindGroup(storageInputs, paramsBuffer, storageOutputs);
        // Get the workgroup counts
        var _d = __read(this.getWorkgroupCounts(env), 3), workgroupCountX = _d[0], workgroupCountY = _d[1], workgroupCountZ = _d[2];
        // Start a new command encoder
        var commandEncoder = this._gpuDevice.createCommandEncoder();
        // Encode the kernel using pass encoder
        var passEncoder = commandEncoder.beginComputePass();
        passEncoder.setPipeline(this._computePipeline);
        passEncoder.setBindGroup(0, bindGroup);
        passEncoder.dispatchWorkgroups(workgroupCountX, workgroupCountY, workgroupCountZ);
        passEncoder.end();
        // Submit GPU commands
        var gpuCommands = commandEncoder.finish();
        this._gpuDevice.queue.submit([gpuCommands]);
        // Return the storage output buffers
        return storageOutputs;
    };
    KernelWebGPU.prototype.getStorageInputBuffer = function (inputSpec, providedInput, inputIndex, env) {
        if (providedInput === null) {
            throw new Error("Missing input buffer #".concat(inputIndex, " (out of ").concat(this.spec.inputs.length, ") named \"").concat(inputSpec.name, "\" in kernel \"").concat(this.key, "\""));
        }
        if (providedInput.usage & GPUBufferUsage.STORAGE) {
            providedInput.unmap();
            return providedInput;
        }
        else {
            throw new Error("Provided input buffer is not a storage buffer");
        }
    };
    KernelWebGPU.prototype.getStorageOutputBuffer = function (outputSpec, providedOutput, outputIndex, env) {
        if (providedOutput !== null) {
            if (providedOutput.usage & GPUBufferUsage.STORAGE) {
                providedOutput.unmap();
                return providedOutput;
            }
            else {
                throw new Error("Provided output buffer is not a storage buffer");
            }
        }
        else {
            var outputElementByteSize = (0, kernel_1.getShaderTypeElementByteSize)(outputSpec.shaderType);
            var outputElementCount = Math.ceil(this._outputSizeFuncs[outputIndex](env));
            // console.log("output size", outputElementCount, outputElementByteSize);
            var outputBufferSize = outputElementByteSize * outputElementCount;
            var outputBuffer = this._gpuDevice.createBuffer({
                mappedAtCreation: false,
                size: outputBufferSize,
                usage: GPUBufferUsage.STORAGE | GPUBufferUsage.COPY_SRC,
            });
            return outputBuffer;
        }
    };
    KernelWebGPU.prototype.createBindGroup = function (inputBuffers, paramsBuffer, outputBuffers) {
        var entries = [];
        var bindingIndex = 0;
        for (var i = 0; i < inputBuffers.length; i++, bindingIndex++) {
            entries.push({
                binding: bindingIndex,
                resource: {
                    buffer: inputBuffers[i],
                },
            });
        }
        for (var i = 0; i < this.spec.outputs.length; i++, bindingIndex++) {
            var outputBuffer = outputBuffers[i];
            entries.push({
                binding: bindingIndex,
                resource: {
                    buffer: outputBuffer,
                },
            });
        }
        entries.push({
            binding: bindingIndex,
            resource: {
                buffer: paramsBuffer,
            },
        });
        var bindGroup = this._gpuDevice.createBindGroup({
            layout: this._bindGroupLayout,
            entries: entries,
        });
        return bindGroup;
    };
    return KernelWebGPU;
}(kernel_1.Kernel));
exports.KernelWebGPU = KernelWebGPU;
//# sourceMappingURL=kernel_webgpu.js.map