"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.kernels = void 0;
exports.kernels = {
    conv2d: {
        name: "conv2d",
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: [
            {
                name: "batchSize",
                shaderType: "u32",
            },
            {
                name: "inputChannels",
                shaderType: "u32",
            },
            {
                name: "outputChannels",
                shaderType: "u32",
            },
            {
                name: "inputHeight",
                shaderType: "u32",
            },
            {
                name: "inputWidth",
                shaderType: "u32",
            },
            {
                name: "kernelHeight",
                shaderType: "u32",
            },
            {
                name: "kernelWidth",
                shaderType: "u32",
            },
            {
                name: "outputHeight",
                shaderType: "u32",
            },
            {
                name: "outputWidth",
                shaderType: "u32",
            },
        ],
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
            {
                name: "weight",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "output",
                shaderType: "array<f32>",
                size: "batchSize * outputChannels * outputHeight * outputWidth",
            },
        ],
        workgroupSize: [4, 4, 1],
        workgroupCount: ["outputWidth/4", "outputHeight/4", 1],
        shader: "\n    if (global_id.x >= parameters.outputWidth || global_id.y >= parameters.outputHeight) {\n        return;\n    }\n    // input shape = [B, C, H, W]\n    for (var batch = 0u; batch < parameters.batchSize; batch++) {\n        for (var outputChannel = 0u; outputChannel < parameters.outputChannels; outputChannel++) {\n            var result = 0.0;\n            // Do the convolution\n            for (var inputChannel = 0u; inputChannel < parameters.inputChannels; inputChannel++) {\n                for (var kernelY = 0u; kernelY < parameters.kernelHeight; kernelY++) {\n                    for (var kernelX = 0u; kernelX < parameters.kernelWidth; kernelX++) {\n                        var inputY = global_id.y + kernelY;\n                        var inputX = global_id.x + kernelX;\n                        var inputIndex =\n                            batch * parameters.inputChannels * parameters.inputHeight * parameters.inputWidth +\n                            inputChannel * parameters.inputHeight * parameters.inputWidth +\n                            inputY * parameters.inputWidth +\n                            inputX;\n                        var kernelIndex =\n                            outputChannel * parameters.inputChannels * parameters.kernelHeight * parameters.kernelWidth +\n                            inputChannel * parameters.kernelHeight * parameters.kernelWidth +\n                            kernelY * parameters.kernelWidth +\n                            kernelX;\n                        result = result + input[inputIndex] * weight[kernelIndex];\n                    }\n                }\n            }\n            // Output\n            let outputIndex = \n                batch * parameters.outputChannels * parameters.outputHeight * parameters.outputWidth +\n                outputChannel * parameters.outputHeight * parameters.outputWidth +\n                global_id.y * parameters.outputWidth +\n                global_id.x;\n            output[outputIndex] = result;\n        }\n    }\n"
    },
    mm: {
        name: "mm",
        config: [
            {
                name: "resultDtype",
            },
        ],
        parameters: [
            {
                name: "resultRows",
                shaderType: "u32",
            },
            {
                name: "resultCols",
                shaderType: "u32",
            },
            {
                name: "innerDim",
                shaderType: "u32",
            },
            {
                name: "alpha",
                shaderType: "f32",
            },
        ],
        inputs: [
            {
                name: "firstMatrix",
                shaderType: "array<f32>",
            },
            {
                name: "secondMatrix",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "resultMatrix",
                shaderType: "array<f32>",
                size: "resultRows * resultCols",
            },
        ],
        workgroupSize: [8, 8, 1],
        workgroupCount: ["resultRows/8", "resultCols/8", 1],
        shader: "\n    if (global_id.x >= parameters.resultRows || global_id.y >= parameters.resultCols) {\n        return;\n    }\n    var result = 0.0;\n    for (var i = 0u; i < parameters.innerDim; i = i + 1u) {\n        let a = global_id.x * parameters.innerDim + i;\n        let b = i * parameters.resultCols + global_id.y;\n        result = result + firstMatrix[a] * secondMatrix[b];\n    }\n    let index = global_id.y + global_id.x * parameters.resultCols;\n    resultMatrix[index] = result;\n"
    },
    sumDim: {
        name: "sumDim",
        config: [
            {
                name: "dtype",
            },
            {
                name: "workgroupSize",
            },
        ],
        parameters: [
            {
                name: "size",
                shaderType: "u32",
            },
        ],
        inputs: [
            {
                name: "dimToSum",
                shaderType: "u32",
            },
            {
                name: "inputShape",
                shaderType: "vec3<u32>",
            },
            {
                name: "inputStrides",
                shaderType: "vec3<u32>",
            },
        ],
        outputs: [
            {
                name: "output",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: ["workgroupSize", "workgroupSize", "workgroupSize"],
        workgroupCount: [1, 1, 1],
        shader: "\n        // Global index flattening for the reformatted 3D tensor\n        var flatGlobalId: u32 = global_id.x * parameters.inputStrides.x + global_id.y * parameters.inputStrides.y + global_id.z * parameters.inputStrides.z;\n    \n        // Initialize sum\n        var sum: f32 = 0.0;\n    \n        let numReductions: u32 = parameters.inputShape.y;\n    \n        // Sum reduction\n        for (var i: u32 = 0; i < numReductions; i = i + 1) {\n            // Compute the input index by adding the reduction offset to the current flat global index\n            var dataIndex: u32 = flatGlobalId + i * parameters.inputStrides.y;\n    \n            if (dataIndex < input.length()) {\n                // Accumulate the input value into sum\n                sum = sum + input[dataIndex];\n            }\n        }\n    \n        // Write the reduced sum value to output tensor\n        if (flatGlobalId < output.length()) {\n            output[flatGlobalId] = sum;\n        }\n    "
    },
};
//# sourceMappingURL=kernels_artisanal.js.map