"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimestepEmbedSequential = exports.UNetModel = void 0;
var nn_cnn_1 = require("./nn_cnn");
var nn_module_1 = require("./nn_module");
var nn_opgen_1 = require("./nn_opgen");
var ops_artisanal_1 = require("./ops_artisanal");
/**
 * Full UNet model with attention and timestep embedding.
 *
 * ** This is still a work in progress **
 */
var UNetModel = /** @class */ (function (_super) {
    __extends(UNetModel, _super);
    /**
     * Constructs a new UNet model with a given number of input and output channels along with
     * a variety of other options.
     */
    function UNetModel(config) {
        var e_1, _a, e_2, _b;
        var _this = this;
        var _c;
        _this = _super.call(this) || this;
        if (config.numHeads === -1) {
            if (config.numHeadChannels === -1) {
                throw new Error("Must specify either numHeads or numHeadChannels");
            }
        }
        if (config.numHeadChannels === -1) {
            if (config.numHeads === -1) {
                throw new Error("Must specify either numHeads or numHeadChannels");
            }
        }
        _this.inChannels = config.inChannels;
        _this.modelChannels = config.modelChannels;
        _this.outChannels = config.outChannels;
        _this.numResBlocks = config.numResBlocks || 2;
        _this.attentionResolutions = config.attentionResolutions || [];
        _this.dropout = config.dropout || 0.0;
        _this.channelMult = config.channelMult || [1, 2, 4, 8];
        _this.convResample =
            config.convResample === undefined ? true : config.convResample;
        _this.numClasses = config.numClasses || null;
        _this.useCheckpoint = config.useCheckpoint || false;
        _this.dtype = config.dtype || "float32";
        _this.numHeadChannels = config.numHeadChannels || -1;
        var contextDim = config.contextDim || null;
        var dims = config.dims || 2;
        var resblockDownup = config.resblockUpdown || false;
        var transformerDepth = config.transformerDepth || 1;
        var useCheckpoint = config.useCheckpoint || false;
        var useNewAttentionOrder = config.useNewAttentionOrder || false;
        var timeEmbedDim = config.modelChannels * 4;
        _this.timeEmbed = new nn_module_1.Sequential([
            linear(_this.modelChannels, timeEmbedDim),
            new nn_opgen_1.SiLU(),
            linear(timeEmbedDim, timeEmbedDim),
        ]);
        _this.inputBlocks = new nn_module_1.ModuleList([
            new TimestepEmbedSequential(conv_nd(dims, _this.inChannels, _this.modelChannels, 3, 1, 1, _this.dtype)),
        ]);
        _this._featureSize = _this.modelChannels;
        var numHeads = config.numHeads || -1;
        var numHeadsUpSample = config.numHeadsUpSample || -1;
        var dimHead = 0;
        // Create the backbone
        var inputBlockChans = [_this.modelChannels];
        var ch = _this.modelChannels;
        var ds = 1;
        try {
            for (var _d = __values(_this.channelMult.entries()), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = __read(_e.value, 2), level = _f[0], mult = _f[1];
                for (var i = 0; i < _this.numResBlocks; i++) {
                    var layers = [
                        new ResBlock(ch, timeEmbedDim, mult * _this.modelChannels, dims, false, false, _this.dropout, _this.useCheckpoint, undefined, config.useScaleShiftNorm),
                    ];
                    ch = mult * _this.modelChannels;
                    if (_this.attentionResolutions.includes(ds)) {
                        if (_this.numHeadChannels === -1) {
                            dimHead = ch / numHeads;
                        }
                        else {
                            numHeads = ch / _this.numHeadChannels;
                            dimHead = _this.numHeadChannels;
                        }
                        if (config.useSpatialTransformer) {
                            layers.push(new SpatialTransformer(ch, numHeads, dimHead, transformerDepth, 0.0, contextDim));
                        }
                        else {
                            layers.push(new AttentionBlock(ch, numHeads, dimHead, useCheckpoint, useNewAttentionOrder));
                        }
                    }
                }
                if (level != _this.channelMult.length - 1) {
                    var outCh = ch;
                    _this.inputBlocks.push(new TimestepEmbedSequential(resblockDownup
                        ? new ResBlock(ch, timeEmbedDim, outCh, dims, false, true, _this.dropout, useCheckpoint, undefined, config.useScaleShiftNorm)
                        : new Downsample(ch, _this.convResample, dims, outCh)));
                    ch = outCh;
                    inputBlockChans.push(ch);
                    ds *= 2;
                    _this._featureSize += ch;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) _a.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // Create the bottleneck
        if (_this.numHeadChannels === -1) {
            dimHead = ch / numHeads;
        }
        else {
            numHeads = ch / _this.numHeadChannels;
            dimHead = _this.numHeadChannels;
        }
        _this.middleBlock = new TimestepEmbedSequential(new ResBlock(ch, timeEmbedDim, ch, dims, false, false, _this.dropout, _this.useCheckpoint, undefined, config.useScaleShiftNorm), config.useSpatialTransformer
            ? new SpatialTransformer(ch, numHeads, dimHead, transformerDepth, 0.0, contextDim)
            : new AttentionBlock(ch, numHeads, dimHead, useCheckpoint, useNewAttentionOrder), new ResBlock(ch, timeEmbedDim, ch, dims, false, false, _this.dropout, _this.useCheckpoint, undefined, config.useScaleShiftNorm));
        _this._featureSize += ch;
        // Output blocks
        _this.outputBlocks = new nn_module_1.ModuleList([]);
        try {
            for (var _g = __values(_this.channelMult.entries()), _h = _g.next(); !_h.done; _h = _g.next()) {
                var _j = __read(_h.value, 2), level = _j[0], mult = _j[1];
                for (var i = 0; i < _this.numResBlocks + 1; i++) {
                    var ich = (_c = inputBlockChans.pop()) !== null && _c !== void 0 ? _c : 0;
                    var layers = [
                        new ResBlock(ch + ich, timeEmbedDim, _this.modelChannels * mult, dims, false, false, _this.dropout, _this.useCheckpoint, undefined, config.useScaleShiftNorm),
                    ];
                    ch = _this.modelChannels * mult;
                    if (_this.attentionResolutions.includes(ds)) {
                        if (_this.numHeadChannels === -1) {
                            dimHead = ch / numHeads;
                        }
                        else {
                            numHeads = ch / _this.numHeadChannels;
                            dimHead = _this.numHeadChannels;
                        }
                        layers.push(config.useSpatialTransformer
                            ? new SpatialTransformer(ch, numHeads, dimHead, transformerDepth, 0.0, contextDim)
                            : new AttentionBlock(ch, numHeadsUpSample, dimHead, useCheckpoint, useNewAttentionOrder));
                    }
                    if (level > 0 && i == _this.numResBlocks) {
                        var outCh = ch;
                        layers.push(resblockDownup
                            ? new ResBlock(ch, timeEmbedDim, outCh, dims, true, false, _this.dropout, _this.useCheckpoint, undefined, config.useScaleShiftNorm)
                            : new Upsample(ch, _this.convResample, dims, outCh));
                        ds /= 2;
                    }
                    _this.outputBlocks.push(new (TimestepEmbedSequential.bind.apply(TimestepEmbedSequential, __spreadArray([void 0], __read(layers), false)))());
                    _this._featureSize += ch;
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_h && !_h.done && (_b = _g.return)) _b.call(_g);
            }
            finally { if (e_2) throw e_2.error; }
        }
        _this.out = new nn_module_1.Sequential([
            normalization(ch),
            new nn_opgen_1.SiLU(),
            zeroModule(conv_nd(dims, _this.modelChannels, _this.outChannels, 3, 1, 1, _this.dtype)),
        ]);
        return _this;
    }
    /**
     * Apply the model to the input batch.
     * @param x a [B, C, ...] Tensor of inputs.
     * @param timesteps a 1-D batch of timesteps.
     * @param context conditioning from cross-attention.
     * @param y a [B] Tensor of labels, if cross-conditional.
     * @returns a [B, C, ...] Tensor of outputs.
     */
    UNetModel.prototype.forward = function (x, timesteps, context, y) {
        var e_3, _a, e_4, _b;
        if ((y !== undefined) != (this.numClasses !== null)) {
            throw new Error("Must specify y if and only if numClasses is set (the model is cross-conditional)");
        }
        var hs = [];
        var tEmb = timestepEmbedding(timesteps, this.modelChannels, this.dtype, false);
        var emb = this.timeEmbed.forward(tEmb);
        if (this.numClasses !== null) {
            if ((y === null || y === void 0 ? void 0 : y.shape.length) != 1 || y.shape[0] != x.shape[0]) {
                throw new Error("y must be a 1-D batch of labels whose batch size matches x");
            }
        }
        var h = x;
        try {
            for (var _c = __values(this.inputBlocks), _d = _c.next(); !_d.done; _d = _c.next()) {
                var module_1 = _d.value;
                h = module_1.forward(h, emb, context);
                hs.push(h);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_3) throw e_3.error; }
        }
        h = this.middleBlock.forward(h, emb, context);
        try {
            for (var _e = __values(this.outputBlocks), _f = _e.next(); !_f.done; _f = _e.next()) {
                var module_2 = _f.value;
                h = (0, ops_artisanal_1.cat)([h, hs.pop()], 1);
                h = module_2.forward(h, emb, context);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return this.out.forward(h);
    };
    return UNetModel;
}(nn_module_1.Module));
exports.UNetModel = UNetModel;
function conv_nd(dims, inChannels, outChannels, kernelSize, stride, padding, dtype) {
    if (dims === 2) {
        return new nn_cnn_1.Conv2d(inChannels, outChannels, kernelSize, stride, padding, dtype);
    }
    else {
        throw new Error("conv_nd: dims ".concat(dims, " not implemented"));
    }
}
function linear(inChannels, outChannels) {
    return new nn_cnn_1.Linear(inChannels, outChannels);
}
function normalization(channels) {
    return new GroupNorm32(32, channels);
}
function timestepEmbedding(timesteps, dim, dtype, repeatOnly, maxPeriod) {
    if (repeatOnly === void 0) { repeatOnly = false; }
    if (maxPeriod === void 0) { maxPeriod = 10000; }
    var embedding;
    if (!repeatOnly) {
        // const half = dim / 2;
        // const freqs =
        //     exp(div(mul(-Math.log(maxPeriod), arange(0, half, dtype)),
        //             half));
        // const args = timesteps[:, None].float() * freqs[None];
        // embedding = cat([cos(args), sin(args)], -1);
        // if (dim % 2 == 1) {
        //     embedding = cat([embedding, zerosLike([embedding.shape[0], 1], dtype)], -1);
        // }
        throw new Error("Not implemented due to missing arange, zerosLike, slicing");
    }
    else {
        // embedding = repeat(timesteps, "b -> b d", {d: dim});
        throw new Error("Not implemented due to missing repeat");
    }
}
function zeroModule(module) {
    var e_5, _a;
    try {
        for (var _b = __values(module.parameters()), _c = _b.next(); !_c.done; _c = _b.next()) {
            var p = _c.value;
            p.detach().zero_();
        }
    }
    catch (e_5_1) { e_5 = { error: e_5_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_5) throw e_5.error; }
    }
    return module;
}
var AttentionBlock = /** @class */ (function (_super) {
    __extends(AttentionBlock, _super);
    function AttentionBlock(channels, numHeads, numHeadChannels, useCheckpoint, useNewAttentionOrder) {
        return _super.call(this) || this;
    }
    return AttentionBlock;
}(nn_module_1.Module));
var Downsample = /** @class */ (function (_super) {
    __extends(Downsample, _super);
    function Downsample(channels, useConv, dims, outChannels, padding) {
        if (dims === void 0) { dims = 2; }
        if (padding === void 0) { padding = 1; }
        return _super.call(this) || this;
    }
    return Downsample;
}(nn_module_1.Module));
var Upsample = /** @class */ (function (_super) {
    __extends(Upsample, _super);
    function Upsample(channels, useConv, dims, outChannels, padding) {
        if (dims === void 0) { dims = 2; }
        if (padding === void 0) { padding = 1; }
        return _super.call(this) || this;
    }
    return Upsample;
}(nn_module_1.Module));
var GroupNorm32 = /** @class */ (function (_super) {
    __extends(GroupNorm32, _super);
    function GroupNorm32(numGroups, channels) {
        return _super.call(this, numGroups, channels) || this;
    }
    return GroupNorm32;
}(nn_cnn_1.GroupNorm));
var SpatialTransformer = /** @class */ (function (_super) {
    __extends(SpatialTransformer, _super);
    function SpatialTransformer(inChannels, nHeads, dHead, depth, dropout, contextDim) {
        if (depth === void 0) { depth = 1; }
        if (dropout === void 0) { dropout = 0; }
        if (contextDim === void 0) { contextDim = null; }
        return _super.call(this) || this;
    }
    return SpatialTransformer;
}(nn_module_1.Module));
var TimestepBlock = /** @class */ (function (_super) {
    __extends(TimestepBlock, _super);
    function TimestepBlock() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return TimestepBlock;
}(nn_module_1.Module));
/**
 * A sequential module that passes timestep embeddings to the children that
 * support it as an extra input.
 */
var TimestepEmbedSequential = /** @class */ (function (_super) {
    __extends(TimestepEmbedSequential, _super);
    function TimestepEmbedSequential() {
        var e_6, _a;
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i] = arguments[_i];
        }
        var _this = _super.call(this) || this;
        try {
            for (var _b = __values(modules.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), i = _d[0], module_3 = _d[1];
                _this.addModule(i, module_3);
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
        return _this;
    }
    TimestepEmbedSequential.prototype.forward = function (x, emb, context) {
        var e_7, _a;
        try {
            for (var _b = __values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                var layer = _c.value;
                if (layer instanceof TimestepBlock) {
                    x = layer.forward(x, emb);
                }
                // else if (layer instanceof SpatialTransformer) {
                //     x = (layer as any)(x, context);
                // }
                else {
                    x = layer(x);
                }
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_7) throw e_7.error; }
        }
        return x;
    };
    return TimestepEmbedSequential;
}(TimestepBlock));
exports.TimestepEmbedSequential = TimestepEmbedSequential;
var ResBlock = /** @class */ (function (_super) {
    __extends(ResBlock, _super);
    function ResBlock(channels, embChannels, outChannels, dims, up, down, dropout, useCheckpoint, useConv, useScaleShiftNorm) {
        if (dims === void 0) { dims = 2; }
        if (up === void 0) { up = false; }
        if (down === void 0) { down = false; }
        if (dropout === void 0) { dropout = 0.0; }
        if (useCheckpoint === void 0) { useCheckpoint = false; }
        if (useConv === void 0) { useConv = false; }
        if (useScaleShiftNorm === void 0) { useScaleShiftNorm = false; }
        return _super.call(this) || this;
    }
    return ResBlock;
}(TimestepBlock));
//# sourceMappingURL=nn_diffusers.js.map