"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tensor = exports.t = exports.mm = exports.conv2d = exports.cat = void 0;
var tensor_1 = require("./tensor");
var autograd_1 = require("./autograd");
function cat(inputs, dim) {
    throw new Error("cat not implemented yet");
}
exports.cat = cat;
/**
 * Applies a 2D convolution over an input image composed of several input planes.
 *
 * #### Forward
 * ```
 * output[y, x] = sum(Ky, sum(Kx, input[y + ky, x + kx] * weight[ky, kx])) + bias
 * ```
 *
 * @param input input tensor of shape [B, inChannels, iH, iW]
 * @param weight filters of shape [outChannels, inChannels, kH, kW]
 * @param bias optional bias tensor of shape [outChannels]
 * @param stride the stride of the convolving kernel. Can be a single number or a tuple (sH, sW). Default: 1
 * @param padding implicit padding on both sides of the kernel. Can be a single number or a tuple (padH, padW). Default: 0
 *     `padding="valid"` is the same as no padding. `padding="same"` pads the input so the output has the shape as the input.
 *     However, this mode can only be used when `stride` is 1.
 * @returns
 */
function conv2d(input, weight, bias, stride, padding) {
    if ((0, autograd_1.shouldCreateGradient)(input, weight)) {
        throw new Error("conv2d gradient not supported yet");
    }
    else {
        if (input.shape.length !== 4 || weight.shape.length !== 4) {
            throw new Error("Expected image tensor, got ".concat(input.shape, " and kernel ").concat(weight.shape));
        }
        if (input.shape[1] !== weight.shape[1]) {
            throw new Error("Expected number of chennels in input image to match number of channels in kernel, got ".concat(input.shape, " and ").concat(weight.shape));
        }
        var params = {
            batchSize: input.shape[0],
            inputChannels: input.shape[1],
            outputChannels: weight.shape[0],
            inputHeight: input.shape[2],
            inputWidth: input.shape[3],
            kernelHeight: weight.shape[2],
            kernelWidth: weight.shape[3],
            outputHeight: input.shape[2] - weight.shape[2] + 1,
            outputWidth: input.shape[3] - weight.shape[3] + 1,
        };
        return input.runKernel("conv2d", { dtype: input.dtype }, params, [[params.batchSize, params.outputChannels, params.outputHeight, params.outputWidth]], weight)[0];
    }
}
exports.conv2d = conv2d;
function mm(input, other) {
    if ((0, autograd_1.shouldCreateGradient)(input, other)) {
        throw new Error("mm gradient not supported yet");
    }
    else {
        if (input.shape.length !== 2 || other.shape.length !== 2) {
            throw new Error("Expected 2D tensors, got ".concat(input.shape, " and ").concat(other.shape));
        }
        if (input.shape[1] !== other.shape[0]) {
            throw new Error("Expected tensors inner dimensions to be compatible, got ".concat(input.shape, " and ").concat(other.shape));
        }
        var params = {
            resultRows: input.shape[0],
            resultCols: other.shape[1],
            innerDim: input.shape[1],
            alpha: 1.0,
        };
        return input.runKernel("mm", { resultDtype: input.dtype }, params, [[params.resultRows, params.resultCols]], other)[0];
    }
}
exports.mm = mm;
function t(input) {
    if (input.shape.length !== 2) {
        throw new Error("Expected 2D tensor, got ".concat(input.shape));
    }
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        throw new Error("t gradient not supported yet");
        // return TransposeFunction.apply(input, 0, 1);
    }
    else {
        var newShape = input.shape.slice();
        newShape.reverse();
        var newStrides = input.strides.slice();
        newStrides.reverse();
        return input.withShape(newShape, newStrides);
    }
}
exports.t = t;
function tensor(arrayOrSpec, dtype, device, requiresGrad) {
    return new tensor_1.Tensor(arrayOrSpec, dtype, device, requiresGrad);
}
exports.tensor = tensor;
//# sourceMappingURL=ops_artisanal.js.map