"use strict";
// High level ops
Object.defineProperty(exports, "__esModule", { value: true });
exports.reduction = exports.binaryWithAlpha = exports.binary = exports.unaryWithAlpha = exports.unary = void 0;
var autograd_1 = require("./autograd");
function unary(func, input) {
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        return func.apply(input);
    }
    return func.forward([input]);
}
exports.unary = unary;
function unaryWithAlpha(func, input, alpha) {
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        return func.apply(input, alpha);
    }
    return func.forward([input, alpha]);
}
exports.unaryWithAlpha = unaryWithAlpha;
function binary(func, input, other) {
    if (input.shape.length !== other.shape.length) {
        throw new Error("Shape dimensions must match. Got ".concat(input.shape, " and ").concat(other.shape));
    }
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        return func.apply(input, other);
    }
    return func.forward([input, other]);
}
exports.binary = binary;
function binaryWithAlpha(func, input, other, alpha) {
    if (input.shape.length !== other.shape.length) {
        throw new Error("Shape dimensions must match. Got ".concat(input.shape, " and ").concat(other.shape));
    }
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        return func.apply(input, other, alpha);
    }
    return func.forward([input, other, alpha]);
}
exports.binaryWithAlpha = binaryWithAlpha;
function reduction(func, input, axis, keepDim) {
    if ((0, autograd_1.shouldCreateGradient)(input)) {
        return func.apply(input, axis, keepDim);
    }
    return func.forward([input, axis, keepDim]);
}
exports.reduction = reduction;
//# sourceMappingURL=ops_high.js.map