"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sin = exports.silu = exports.sign = exports.sigmoid = exports.rsqrt = exports.round = exports.relu = exports.reciprocal = exports.rad2deg = exports.pow = exports.positive = exports.negative = exports.neg = exports.multiply = exports.mul = exports.logaddexp2 = exports.logaddexp = exports.log2 = exports.log1p = exports.log10 = exports.log = exports.ldexp = exports.hypot = exports.frac = exports.floor = exports.expm1 = exports.exp2 = exports.exp = exports.divide = exports.div = exports.deg2rad = exports.cosh = exports.cos = exports.copysign = exports.ceil = exports.arctan2 = exports.atan2 = exports.arctan = exports.atan = exports.arcsinh = exports.asinh = exports.arcsin = exports.asin = exports.add = exports.arccosh = exports.acosh = exports.arccos = exports.acos = exports.absolute = exports.abs = void 0;
exports.countNonzero = exports.sum = exports.prod = exports.norm = exports.mean = exports.any = exports.all = exports.xlogy = exports.fix = exports.trunc = exports.tanh = exports.tan = exports.subtract = exports.sub = exports.square = exports.sqrt = exports.sinh = exports.sinc = void 0;
var functions = require("./functions_opgen");
var ops_high_1 = require("./ops_high");
/**
* ![Plot of abs and its gradient](/plots/abs.svg)
*
* Calculates:
* ```js
* output = abs(input)
* ```
*
* Gradient:
* ```js
* inputGrad = input == 0 ? 0 : (input > 0 ? outputGrad : -outputGrad)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function abs(input) {
    return (0, ops_high_1.unary)(functions.AbsFunction, input);
}
exports.abs = abs;
/**
* Alias for `abs`.
*
* ![Plot of abs and its gradient](/plots/abs.svg)
*
* Calculates:
* ```js
* output = abs(input)
* ```
*
* Gradient:
* ```js
* inputGrad = input == 0 ? 0 : (input > 0 ? outputGrad : -outputGrad)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function absolute(input) {
    return abs(input);
}
exports.absolute = absolute;
/**
* ![Plot of acos and its gradient](/plots/acos.svg)
*
* Calculates:
* ```js
* output = acos(input)
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad / sqrt(1 - input * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function acos(input) {
    return (0, ops_high_1.unary)(functions.AcosFunction, input);
}
exports.acos = acos;
/**
* Alias for `acos`.
*
* ![Plot of acos and its gradient](/plots/acos.svg)
*
* Calculates:
* ```js
* output = acos(input)
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad / sqrt(1 - input * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function arccos(input) {
    return acos(input);
}
exports.arccos = arccos;
/**
* ![Plot of acosh and its gradient](/plots/acosh.svg)
*
* Calculates:
* ```js
* output = acosh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(input * input - 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function acosh(input) {
    return (0, ops_high_1.unary)(functions.AcoshFunction, input);
}
exports.acosh = acosh;
/**
* Alias for `acosh`.
*
* ![Plot of acosh and its gradient](/plots/acosh.svg)
*
* Calculates:
* ```js
* output = acosh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(input * input - 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function arccosh(input) {
    return acosh(input);
}
exports.arccosh = arccosh;
/**
* Calculates:
* ```js
* output = input + other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad; otherGrad = outputGrad
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function add(input, other, alpha) {
    return (0, ops_high_1.binaryWithAlpha)(functions.AddFunction, input, other, alpha);
}
exports.add = add;
/**
* ![Plot of asin and its gradient](/plots/asin.svg)
*
* Calculates:
* ```js
* output = asin(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(1 - input * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function asin(input) {
    return (0, ops_high_1.unary)(functions.AsinFunction, input);
}
exports.asin = asin;
/**
* Alias for `asin`.
*
* ![Plot of asin and its gradient](/plots/asin.svg)
*
* Calculates:
* ```js
* output = asin(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(1 - input * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function arcsin(input) {
    return asin(input);
}
exports.arcsin = arcsin;
/**
* ![Plot of asinh and its gradient](/plots/asinh.svg)
*
* Calculates:
* ```js
* output = asinh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(input * input + 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function asinh(input) {
    return (0, ops_high_1.unary)(functions.AsinhFunction, input);
}
exports.asinh = asinh;
/**
* Alias for `asinh`.
*
* ![Plot of asinh and its gradient](/plots/asinh.svg)
*
* Calculates:
* ```js
* output = asinh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / sqrt(input * input + 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function arcsinh(input) {
    return asinh(input);
}
exports.arcsinh = arcsinh;
/**
* ![Plot of atan and its gradient](/plots/atan.svg)
*
* Calculates:
* ```js
* output = atan(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (input * input + 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function atan(input) {
    return (0, ops_high_1.unary)(functions.AtanFunction, input);
}
exports.atan = atan;
/**
* Alias for `atan`.
*
* ![Plot of atan and its gradient](/plots/atan.svg)
*
* Calculates:
* ```js
* output = atan(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (input * input + 1)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function arctan(input) {
    return atan(input);
}
exports.arctan = arctan;
/**
* Calculates:
* ```js
* output = atan2(input, other)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * other / (input * input + other * other); otherGrad = -outputGrad * input / (input * input + other * other)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function atan2(input, other) {
    return (0, ops_high_1.binary)(functions.Atan2Function, input, other);
}
exports.atan2 = atan2;
/**
* Alias for `atan2`.
*
* Calculates:
* ```js
* output = atan2(input, other)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * other / (input * input + other * other); otherGrad = -outputGrad * input / (input * input + other * other)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function arctan2(input, other) {
    return atan2(input, other);
}
exports.arctan2 = arctan2;
/**
* ![Plot of ceil and its gradient](/plots/ceil.svg)
*
* Calculates:
* ```js
* output = ceil(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function ceil(input) {
    return (0, ops_high_1.unary)(functions.CeilFunction, input);
}
exports.ceil = ceil;
/**
* Calculates:
* ```js
* output = other >= 0 ? abs(input) : -abs(input)
* ```
*
* Gradient:
* ```js
* var dir = other >= 0 ? (input >= 0 ? 1.0 : -1.0) : (input >= 0 ? -1.0 : 1.0); inputGrad = input == 0.0 ? 0.0 : outputGrad * dir; otherGrad = 0
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function copysign(input, other) {
    return (0, ops_high_1.binary)(functions.CopysignFunction, input, other);
}
exports.copysign = copysign;
/**
* ![Plot of cos and its gradient](/plots/cos.svg)
*
* Calculates:
* ```js
* output = cos(input)
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad * sin(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function cos(input) {
    return (0, ops_high_1.unary)(functions.CosFunction, input);
}
exports.cos = cos;
/**
* ![Plot of cosh and its gradient](/plots/cosh.svg)
*
* Calculates:
* ```js
* output = cosh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * sinh(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function cosh(input) {
    return (0, ops_high_1.unary)(functions.CoshFunction, input);
}
exports.cosh = cosh;
/**
* ![Plot of deg2rad and its gradient](/plots/deg2rad.svg)
*
* Calculates:
* ```js
* output = input * 0.017453292519943295
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * 0.017453292519943295
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function deg2rad(input) {
    return (0, ops_high_1.unary)(functions.Deg2radFunction, input);
}
exports.deg2rad = deg2rad;
/**
* Calculates:
* ```js
* output = input / other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / other; otherGrad = -outputGrad * input / (other * other)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function div(input, other, alpha) {
    return (0, ops_high_1.binaryWithAlpha)(functions.DivFunction, input, other, alpha);
}
exports.div = div;
/**
* Alias for `div`.
*
* Calculates:
* ```js
* output = input / other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / other; otherGrad = -outputGrad * input / (other * other)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function divide(input, other, alpha) {
    return div(input, other, alpha);
}
exports.divide = divide;
/**
* ![Plot of exp and its gradient](/plots/exp.svg)
*
* Calculates:
* ```js
* output = exp(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * exp(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function exp(input) {
    return (0, ops_high_1.unary)(functions.ExpFunction, input);
}
exports.exp = exp;
/**
* ![Plot of exp2 and its gradient](/plots/exp2.svg)
*
* Calculates:
* ```js
* output = exp2(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * exp2(input) * 0.6931471805599453
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function exp2(input) {
    return (0, ops_high_1.unary)(functions.Exp2Function, input);
}
exports.exp2 = exp2;
/**
* ![Plot of expm1 and its gradient](/plots/expm1.svg)
*
* Calculates:
* ```js
* output = exp(input) - 1.0
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * exp(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function expm1(input) {
    return (0, ops_high_1.unary)(functions.Expm1Function, input);
}
exports.expm1 = expm1;
/**
* ![Plot of floor and its gradient](/plots/floor.svg)
*
* Calculates:
* ```js
* output = floor(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function floor(input) {
    return (0, ops_high_1.unary)(functions.FloorFunction, input);
}
exports.floor = floor;
/**
* ![Plot of frac and its gradient](/plots/frac.svg)
*
* Calculates:
* ```js
* output = input >= 0.0 ? fract(input) : -fract(-input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function frac(input) {
    return (0, ops_high_1.unary)(functions.FracFunction, input);
}
exports.frac = frac;
/**
* Calculates:
* ```js
* output = sqrt(input * input + other * other)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * input / sqrt(input * input + other * other); otherGrad = outputGrad * other / sqrt(input * input + other * other)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function hypot(input, other) {
    return (0, ops_high_1.binary)(functions.HypotFunction, input, other);
}
exports.hypot = hypot;
/**
* Calculates:
* ```js
* output = input * pow(2.0, other)
* ```
*
* Gradient:
* ```js
* var out = pow(2.0, other); inputGrad = outputGrad * out; otherGrad = outputGrad * input * out * 0.6931471805599453
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function ldexp(input, other) {
    return (0, ops_high_1.binary)(functions.LdexpFunction, input, other);
}
exports.ldexp = ldexp;
/**
* ![Plot of log and its gradient](/plots/log.svg)
*
* Calculates:
* ```js
* output = log(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / input
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function log(input) {
    return (0, ops_high_1.unary)(functions.LogFunction, input);
}
exports.log = log;
/**
* ![Plot of log10 and its gradient](/plots/log10.svg)
*
* Calculates:
* ```js
* output = log(input) * 0.4342944819032518
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (input * 2.302585092994046)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function log10(input) {
    return (0, ops_high_1.unary)(functions.Log10Function, input);
}
exports.log10 = log10;
/**
* ![Plot of log1p and its gradient](/plots/log1p.svg)
*
* Calculates:
* ```js
* output = log(input + 1.0)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (input + 1.0)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function log1p(input) {
    return (0, ops_high_1.unary)(functions.Log1pFunction, input);
}
exports.log1p = log1p;
/**
* ![Plot of log2 and its gradient](/plots/log2.svg)
*
* Calculates:
* ```js
* output = log2(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (input * 0.6931471805599453)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function log2(input) {
    return (0, ops_high_1.unary)(functions.Log2Function, input);
}
exports.log2 = log2;
/**
* Calculates:
* ```js
* output = log(exp(input) + exp(other))
* ```
*
* Gradient:
* ```js
* var ein = exp(input); var eoth = exp(other); var addeinv = outputGrad/(ein + eoth); inputGrad = addeinv * ein; otherGrad = addeinv * eoth
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function logaddexp(input, other) {
    return (0, ops_high_1.binary)(functions.LogaddexpFunction, input, other);
}
exports.logaddexp = logaddexp;
/**
* Calculates:
* ```js
* output = log2(exp2(input) + exp2(other))
* ```
*
* Gradient:
* ```js
* var ein = exp2(input); var eoth = exp2(other); var sum_ein_eoth = ein + eoth; inputGrad = outputGrad * (ein / sum_ein_eoth); otherGrad = outputGrad * (eoth / sum_ein_eoth );
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function logaddexp2(input, other) {
    return (0, ops_high_1.binary)(functions.Logaddexp2Function, input, other);
}
exports.logaddexp2 = logaddexp2;
/**
* Calculates:
* ```js
* output = input * other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * other; otherGrad = outputGrad * input
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function mul(input, other, alpha) {
    return (0, ops_high_1.binaryWithAlpha)(functions.MulFunction, input, other, alpha);
}
exports.mul = mul;
/**
* Alias for `mul`.
*
* Calculates:
* ```js
* output = input * other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * other; otherGrad = outputGrad * input
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function multiply(input, other, alpha) {
    return mul(input, other, alpha);
}
exports.multiply = multiply;
/**
* ![Plot of neg and its gradient](/plots/neg.svg)
*
* Calculates:
* ```js
* output = -input
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function neg(input) {
    return (0, ops_high_1.unary)(functions.NegFunction, input);
}
exports.neg = neg;
/**
* Alias for `neg`.
*
* ![Plot of neg and its gradient](/plots/neg.svg)
*
* Calculates:
* ```js
* output = -input
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function negative(input) {
    return neg(input);
}
exports.negative = negative;
/**
* ![Plot of positive and its gradient](/plots/positive.svg)
*
* Calculates:
* ```js
* output = input
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function positive(input) {
    return (0, ops_high_1.unary)(functions.PositiveFunction, input);
}
exports.positive = positive;
/**
* Calculates:
* ```js
* output = pow(input, other)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * other * pow(input, other - 1.0); otherGrad = outputGrad * pow(input, other) * log(input)
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function pow(input, other) {
    return (0, ops_high_1.binary)(functions.PowFunction, input, other);
}
exports.pow = pow;
/**
* ![Plot of rad2deg and its gradient](/plots/rad2deg.svg)
*
* Calculates:
* ```js
* output = input * 57.29577951308232
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * 57.29577951308232
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function rad2deg(input) {
    return (0, ops_high_1.unary)(functions.Rad2degFunction, input);
}
exports.rad2deg = rad2deg;
/**
* ![Plot of reciprocal and its gradient](/plots/reciprocal.svg)
*
* Calculates:
* ```js
* output = 1.0 / input
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad / (input * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function reciprocal(input) {
    return (0, ops_high_1.unary)(functions.ReciprocalFunction, input);
}
exports.reciprocal = reciprocal;
/**
* ![Plot of relu and its gradient](/plots/relu.svg)
*
* Calculates:
* ```js
* output = max(input, 0.0)
* ```
*
* Gradient:
* ```js
* inputGrad = input > 0.0 ? outputGrad : 0.0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function relu(input) {
    return (0, ops_high_1.unary)(functions.ReluFunction, input);
}
exports.relu = relu;
/**
* ![Plot of round and its gradient](/plots/round.svg)
*
* Calculates:
* ```js
* output = round(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function round(input) {
    return (0, ops_high_1.unary)(functions.RoundFunction, input);
}
exports.round = round;
/**
* ![Plot of rsqrt and its gradient](/plots/rsqrt.svg)
*
* Calculates:
* ```js
* output = 1.0 / sqrt(input)
* ```
*
* Gradient:
* ```js
* inputGrad = -outputGrad / (2.0 * sqrt(input) * input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function rsqrt(input) {
    return (0, ops_high_1.unary)(functions.RsqrtFunction, input);
}
exports.rsqrt = rsqrt;
/**
* ![Plot of sigmoid and its gradient](/plots/sigmoid.svg)
*
* Calculates:
* ```js
* output = 1.0 / (1.0 + exp(-input))
* ```
*
* Gradient:
* ```js
* var out = 1.0 / (1.0 + exp(-input)); inputGrad = outputGrad * out * (1.0 - out)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sigmoid(input) {
    return (0, ops_high_1.unary)(functions.SigmoidFunction, input);
}
exports.sigmoid = sigmoid;
/**
* ![Plot of sign and its gradient](/plots/sign.svg)
*
* Calculates:
* ```js
* output = sign(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sign(input) {
    return (0, ops_high_1.unary)(functions.SignFunction, input);
}
exports.sign = sign;
/**
* ![Plot of silu and its gradient](/plots/silu.svg)
*
* Calculates:
* ```js
* output = input / (1.0 + exp(-input))
* ```
*
* Gradient:
* ```js
* var out = 1.0 / (1.0 + exp(-input)); inputGrad = outputGrad * (out + input * out * (1.0 - out))
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function silu(input) {
    return (0, ops_high_1.unary)(functions.SiluFunction, input);
}
exports.silu = silu;
/**
* ![Plot of sin and its gradient](/plots/sin.svg)
*
* Calculates:
* ```js
* output = sin(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * cos(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sin(input) {
    return (0, ops_high_1.unary)(functions.SinFunction, input);
}
exports.sin = sin;
/**
* ![Plot of sinc and its gradient](/plots/sinc.svg)
*
* Calculates:
* ```js
* var inpi = input * 3.141592653589793; output = input == 0.0 ? 1.0 : sin(inpi) / inpi
* ```
*
* Gradient:
* ```js
* var inpi = input * 3.141592653589793; inputGrad = input == 0.0 ? 0.0 : (outputGrad * 3.141592653589793 * (inpi*cos(inpi) - sin(inpi)) / (inpi*inpi))
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sinc(input) {
    return (0, ops_high_1.unary)(functions.SincFunction, input);
}
exports.sinc = sinc;
/**
* ![Plot of sinh and its gradient](/plots/sinh.svg)
*
* Calculates:
* ```js
* output = sinh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * cosh(input)
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sinh(input) {
    return (0, ops_high_1.unary)(functions.SinhFunction, input);
}
exports.sinh = sinh;
/**
* ![Plot of sqrt and its gradient](/plots/sqrt.svg)
*
* Calculates:
* ```js
* output = sqrt(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (2.0 * sqrt(input))
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sqrt(input) {
    return (0, ops_high_1.unary)(functions.SqrtFunction, input);
}
exports.sqrt = sqrt;
/**
* ![Plot of square and its gradient](/plots/square.svg)
*
* Calculates:
* ```js
* output = input * input
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * 2.0 * input
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function square(input) {
    return (0, ops_high_1.unary)(functions.SquareFunction, input);
}
exports.square = square;
/**
* Calculates:
* ```js
* output = input - other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad; otherGrad = -outputGrad
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function sub(input, other, alpha) {
    return (0, ops_high_1.binaryWithAlpha)(functions.SubFunction, input, other, alpha);
}
exports.sub = sub;
/**
* Alias for `sub`.
*
* Calculates:
* ```js
* output = input - other
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad; otherGrad = -outputGrad
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @param alpha the alpha value to multiply `other` with
* @returns the output tensor
*/
function subtract(input, other, alpha) {
    return sub(input, other, alpha);
}
exports.subtract = subtract;
/**
* ![Plot of tan and its gradient](/plots/tan.svg)
*
* Calculates:
* ```js
* output = tan(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad / (cos(input) * cos(input))
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function tan(input) {
    return (0, ops_high_1.unary)(functions.TanFunction, input);
}
exports.tan = tan;
/**
* ![Plot of tanh and its gradient](/plots/tanh.svg)
*
* Calculates:
* ```js
* output = tanh(input)
* ```
*
* Gradient:
* ```js
* inputGrad = outputGrad * (1.0 - tanh(input) * tanh(input))
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function tanh(input) {
    return (0, ops_high_1.unary)(functions.TanhFunction, input);
}
exports.tanh = tanh;
/**
* ![Plot of trunc and its gradient](/plots/trunc.svg)
*
* Calculates:
* ```js
* output = trunc(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function trunc(input) {
    return (0, ops_high_1.unary)(functions.TruncFunction, input);
}
exports.trunc = trunc;
/**
* Alias for `trunc`.
*
* ![Plot of trunc and its gradient](/plots/trunc.svg)
*
* Calculates:
* ```js
* output = trunc(input)
* ```
*
* Gradient:
* ```js
* inputGrad = 0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function fix(input) {
    return trunc(input);
}
exports.fix = fix;
/**
* Calculates:
* ```js
* output = input == 0.0 ? 0.0 : input * log(other)
* ```
*
* Gradient:
* ```js
* inputGrad = input == 0.0 ? 0.0 : outputGrad * log(other); otherGrad = input == 0.0 ? 0.0 : outputGrad * (input / other);
* ```
*
* @param input the input tensor of any shape
* @param other the other tensor whose shape is broadcastable with the input tensor
* @returns the output tensor
*/
function xlogy(input, other) {
    return (0, ops_high_1.binary)(functions.XlogyFunction, input, other);
}
exports.xlogy = xlogy;
/**
* Calculates:
* ```js
* output = output && input
* ```
*
* with an initial value of `output = 1`.
*
* Gradient:
* ```js
* inputGrad = output ? outputGrad : 0.0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function all(input) {
    return (0, ops_high_1.unary)(functions.AllFunction, input);
}
exports.all = all;
/**
* Calculates:
* ```js
* output = output || input
* ```
*
* with an initial value of `output = 0`.
*
* Gradient:
* ```js
* inputGrad = output ? outputGrad : 0.0
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function any(input) {
    return (0, ops_high_1.unary)(functions.AnyFunction, input);
}
exports.any = any;
/**
* Calculates:
* ```js
* output = output + input
* ```
*
* with an initial value of `output = 0.0`.
*
* Gradient:
* ```js
* inputGrad = outputGrad / inputSize
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function mean(input) {
    return (0, ops_high_1.unary)(functions.MeanFunction, input);
}
exports.mean = mean;
/**
* Calculates:
* ```js
* output = output + input * input
* ```
*
* with an initial value of `output = 0.0`.
*
* Gradient:
* ```js
* inputGrad = outputGrad * input / output
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function norm(input) {
    return (0, ops_high_1.unary)(functions.NormFunction, input);
}
exports.norm = norm;
/**
* Calculates:
* ```js
* output = output * input
* ```
*
* with an initial value of `output = 1.0`.
*
* Gradient:
* ```js
* inputGrad = outputGrad * output / input
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function prod(input) {
    return (0, ops_high_1.unary)(functions.ProdFunction, input);
}
exports.prod = prod;
/**
* Calculates:
* ```js
* output = output + input
* ```
*
* with an initial value of `output = 0.0`.
*
* Gradient:
* ```js
* inputGrad = outputGrad
* ```
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function sum(input) {
    return (0, ops_high_1.unary)(functions.SumFunction, input);
}
exports.sum = sum;
/**
* Calculates:
* ```js
* output = output + (input != 0)
* ```
*
* with an initial value of `output = 0.0`.
*
* @param input the input tensor of any shape
* @returns the output tensor
*/
function countNonzero(input) {
    return (0, ops_high_1.unary)(functions.CountNonzeroFunction, input);
}
exports.countNonzero = countNonzero;
//# sourceMappingURL=ops_opgen.js.map